@@ -18,10 +18,10 @@
 
 using namespace KNSCore;
 
-class EntryInternal::Private : public QSharedData
+class KNSCore::EntryInternalPrivate : public QSharedData
 {
 public:
-    Private()
+    EntryInternalPrivate()
         : mReleaseDate(QDate::currentDate())
         , mRating(0)
         , mNumberOfComments(0)
@@ -30,11 +30,12 @@ public:
         , mNumberKnowledgebaseEntries(0)
         , mStatus(KNS3::Entry::Invalid)
         , mSource(EntryInternal::Online)
+        , mEntryType(EntryInternal::CatalogEntry)
     {
         qRegisterMetaType<KNSCore::EntryInternal::List>();
     }
 
-    bool operator==(const Private &other) const
+    bool operator==(const EntryInternalPrivate &other) const
     {
         return mUniqueId == other.mUniqueId && mProviderId == other.mProviderId;
     }
@@ -72,6 +73,7 @@ public:
     QString mSignature;
     KNS3::Entry::Status mStatus;
     EntryInternal::Source mSource;
+    EntryInternal::EntryType mEntryType;
 
     QString mPreviewUrl[6];
     QImage mPreviewImage[6];
@@ -80,7 +82,7 @@ public:
 };
 
 EntryInternal::EntryInternal()
-    : d(new Private)
+    : d(new EntryInternalPrivate())
 {
 }
 
@@ -367,6 +369,16 @@ EntryInternal::Source EntryInternal::source() const
     return d->mSource;
 }
 
+void EntryInternal::setEntryType(EntryInternal::EntryType type)
+{
+    d->mEntryType = type;
+}
+
+EntryInternal::EntryType EntryInternal::entryType() const
+{
+    return d->mEntryType;
+}
+
 void EntryInternal::setSource(Source source)
 {
     d->mSource = source;
@@ -430,13 +442,13 @@ static QXmlStreamReader::TokenType readNextSkipComments(QXmlStreamReader *xml)
     return xml->tokenType();
 }
 
-static QStringRef readText(QXmlStreamReader *xml)
+static QString readText(QXmlStreamReader *xml)
 {
     Q_ASSERT(xml->tokenType() == QXmlStreamReader::StartElement);
-    QStringRef ret;
+    QString ret;
     const auto token = readNextSkipComments(xml);
     if (token == QXmlStreamReader::Characters) {
-        ret = xml->text();
+        ret = xml->text().toString();
     }
     return ret;
 }
@@ -444,10 +456,11 @@ static QStringRef readText(QXmlStreamReader *xml)
 static QString readStringTrimmed(QXmlStreamReader *xml)
 {
     Q_ASSERT(xml->tokenType() == QXmlStreamReader::StartElement);
-    QString ret = readText(xml).trimmed().toString();
+    QString ret = readText(xml).trimmed();
 
-    if (xml->tokenType() == QXmlStreamReader::Characters)
+    if (xml->tokenType() == QXmlStreamReader::Characters) {
         readNextSkipComments(xml);
+    }
     Q_ASSERT(xml->tokenType() == QXmlStreamReader::EndElement);
     return ret;
 }
@@ -473,22 +486,25 @@ bool KNSCore::EntryInternal::setEntryXML(QXmlStreamReader &reader)
 
     while (!reader.atEnd()) {
         const auto token = readNextSkipComments(&reader);
-        if (token == QXmlStreamReader::EndElement)
+        if (token == QXmlStreamReader::EndElement) {
             break;
-        else if (token != QXmlStreamReader::StartElement)
+        } else if (token != QXmlStreamReader::StartElement) {
             continue;
+        }
 
         if (reader.name() == QLatin1String("name")) {
             // TODO maybe do something with the language attribute? QString lang = e.attribute("lang");
             d->mName = reader.readElementText(QXmlStreamReader::SkipChildElements);
         } else if (reader.name() == QLatin1String("author")) {
+            // ### careful, the following variables are string views that become invalid when we
+            // proceed with reading from reader, such as the readStringTrimmed call below does!
             const auto email = reader.attributes().value(QStringLiteral("email"));
             const auto jabber = reader.attributes().value(QStringLiteral("im"));
             const auto homepage = reader.attributes().value(QStringLiteral("homepage"));
-            d->mAuthor.setName(readStringTrimmed(&reader));
             d->mAuthor.setEmail(email.toString());
             d->mAuthor.setJabber(jabber.toString());
             d->mAuthor.setHomepage(homepage.toString());
+            d->mAuthor.setName(readStringTrimmed(&reader));
         } else if (reader.name() == QLatin1String("providerid")) {
             d->mProviderId = reader.readElementText(QXmlStreamReader::SkipChildElements);
         } else if (reader.name() == QLatin1String("homepage")) {
@@ -534,8 +550,9 @@ bool KNSCore::EntryInternal::setEntryXML(QXmlStreamReader &reader)
             } else if (statusText == QLatin1String("updateable")) {
                 d->mStatus = KNS3::Entry::Updateable;
             }
-            if (reader.tokenType() == QXmlStreamReader::Characters)
+            if (reader.tokenType() == QXmlStreamReader::Characters) {
                 readNextSkipComments(&reader);
+            }
         }
         Q_ASSERT_X(reader.tokenType() == QXmlStreamReader::EndElement,
                    Q_FUNC_INFO,
@@ -701,7 +718,7 @@ QDomElement KNSCore::EntryInternal::entryXML() const
     if (!d->mChecksum.isEmpty()) {
         (void)addElement(doc, el, QStringLiteral("checksum"), d->mChecksum);
     }
-    for (const QString &file : qAsConst(d->mInstalledFiles)) {
+    for (const QString &file : std::as_const(d->mInstalledFiles)) {
         (void)addElement(doc, el, QStringLiteral("installedfile"), file);
     }
     if (!d->mUniqueId.isEmpty()) {
